VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsVoiceInformation"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 11/2022

Option Explicit

' ----==== Const ====----
Private Const S_OK As Long = &H0&
Private Const CC_STDCALL As Long = &H4&

' ----==== Enums ====----
Private Enum vtb_Interfaces
    
    ' IUnknown
    IUnknown_Release = 2
    
    ' IVoiceInformation
    IVoiceInformation_GetDisplayName = 6
    IVoiceInformation_GetId = 7
    IVoiceInformation_GetLanguage = 8
    IVoiceInformation_GetDescription = 9
    IVoiceInformation_GetGender = 10
    
End Enum

Public Enum VoiceGender
    VoiceGender_Male = 0
    VoiceGender_Female = 1
End Enum

' ----==== Types ====----

' ----==== Ole32.dll Declarations ====----
Private Declare Sub CoTaskMemFree Lib "ole32.dll" ( _
                    ByVal hMem As Long)
                         
' ----==== Oleaut32.dll Declarations ====----
Private Declare Function DispCallFunc Lib "Oleaut32.dll" ( _
                         ByVal pvInstance As Long, _
                         ByVal oVft As Long, _
                         ByVal cc As Long, _
                         ByVal vtReturn As VbVarType, _
                         ByVal cActuals As Long, _
                         ByVal prgvt As Any, _
                         ByVal prgpvarg As Any, _
                         ByRef pvargResult As Variant) As Long

' ----==== Kernel32.dll Declarations ====----
Private Declare Sub CopyMemory Lib "kernel32.dll" _
                    Alias "RtlMoveMemory" ( _
                    ByRef pTo As Any, _
                    ByRef uFrom As Any, _
                    ByVal lSize As Long)

Private Declare Function lstrlenW Lib "kernel32.dll" ( _
                         ByVal lpString As Long) As Long

' ----==== Combase.dll Declarations ====----
Private Declare Function WindowsGetStringRawBuffer Lib "Combase.dll" ( _
                         ByVal sourceString As Long, _
                         ByRef length As Long) As Long

' ----==== Variablen ====----
Private m_pIVoiceInformation As Long


' ----==== Class ====----
Private Sub Class_Initialize()
    '
End Sub

Private Sub Class_Terminate()
    Call Release(m_pIVoiceInformation)
End Sub

' ----==== Properties ====----
Public Property Let IVoiceInformation(ByVal pIVoiceInformation As Long)
    Call Release(m_pIVoiceInformation)
    m_pIVoiceInformation = pIVoiceInformation
End Property

Public Property Get IVoiceInformation() As Long
    IVoiceInformation = m_pIVoiceInformation
End Property

Public Property Get DisplayName() As String
    Dim strRet As String
    If m_pIVoiceInformation <> 0& Then
        Dim hValue As Long
        If Invoke(m_pIVoiceInformation, _
                  IVoiceInformation_GetDisplayName, _
                  VarPtr(hValue)) = S_OK Then
            strRet = Ptr2Str(WindowsGetStringRawBuffer(hValue, 0&))
        End If
    End If
    DisplayName = strRet
End Property

Public Property Get Id() As String
    Dim strRet As String
    If m_pIVoiceInformation <> 0& Then
        Dim hValue As Long
        If Invoke(m_pIVoiceInformation, _
                  IVoiceInformation_GetId, _
                  VarPtr(hValue)) = S_OK Then
            strRet = Ptr2Str(WindowsGetStringRawBuffer(hValue, 0&))
        End If
    End If
    Id = strRet
End Property

Public Property Get Language() As String
    Dim strRet As String
    If m_pIVoiceInformation <> 0& Then
        Dim hValue As Long
        If Invoke(m_pIVoiceInformation, _
                  IVoiceInformation_GetLanguage, _
                  VarPtr(hValue)) = S_OK Then
            strRet = Ptr2Str(WindowsGetStringRawBuffer(hValue, 0&))
        End If
    End If
    Language = strRet
End Property

Public Property Get Description() As String
    Dim strRet As String
    If m_pIVoiceInformation <> 0& Then
        Dim hValue As Long
        If Invoke(m_pIVoiceInformation, _
                  IVoiceInformation_GetDescription, _
                  VarPtr(hValue)) = S_OK Then
            strRet = Ptr2Str(WindowsGetStringRawBuffer(hValue, 0&))
        End If
    End If
    Description = strRet
End Property

Public Property Get Gender() As VoiceGender
    Dim eRet As VoiceGender
    If m_pIVoiceInformation <> 0& Then
        Dim lngValue As Long
        If Invoke(m_pIVoiceInformation, _
                  IVoiceInformation_GetGender, _
                  VarPtr(lngValue)) = S_OK Then
            eRet = lngValue
        End If
    End If
    Gender = eRet
End Property

' ----==== Subs & Functions ====----
Private Sub Release(ByRef pInterface As Long)
    Dim lngRet As Long
    If pInterface <> 0& Then
        Call Invoke(pInterface, IUnknown_Release)
        pInterface = 0&
    End If
End Sub

Private Function Invoke(ByVal pInterface As Long, _
                        ByVal vtb As vtb_Interfaces, _
                        ParamArray var()) As Variant
    If pInterface <> 0& Then
        Invoke = OleInvoke(pInterface, vtb, var)
    End If
End Function

Private Function OleInvoke(ByVal pInterface As Long, _
                           ByVal lngCmd As Long, _
                           ParamArray aParam()) As Variant
    If pInterface <> 0& Then
        If UBound(aParam) >= 0& Then
            Dim varParam As Variant
            varParam = aParam
            If IsArray(varParam) Then varParam = varParam(0)
            Dim lngCount As Long
            lngCount = UBound(varParam)
            Dim lngItem As Long
            Dim olePtr(10) As Long
            Dim oleTyp(10) As Integer
            For lngItem = 0& To lngCount
                oleTyp(lngItem) = VarType(varParam(lngItem))
                olePtr(lngItem) = VarPtr(varParam(lngItem))
            Next
        End If
        Dim varRet As Variant
        If DispCallFunc(pInterface, lngCmd * 4, CC_STDCALL, _
                        vbLong, lngItem, VarPtr(oleTyp(0)), _
                        VarPtr(olePtr(0)), varRet) <> S_OK Then
            Debug.Print "Fehler beim Aufrufen der Interface-Funktion!"
        End If

'        If varRet <> S_OK Then
'            Debug.Print "0x" & Hex$(varRet)
'        End If
    End If
    OleInvoke = varRet
End Function

Private Function Ptr2Str(ByVal lpStr As Long) As String
    Dim lngLen As Long
    Dim bytBuffer() As Byte
    lngLen = lstrlenW(lpStr) * 2
    If lngLen > 0 Then
        ReDim bytBuffer(lngLen - 1)
        Call CopyMemory(bytBuffer(0), ByVal lpStr, lngLen)
        Call CoTaskMemFree(lpStr)
        Ptr2Str = bytBuffer
    End If
End Function
